<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'EMSG_Company_Plugin' ) ) :

class EMSG_Company_Plugin {

	static $sprefix = '_emsg_company_';

	private $type = 'emsg_company';
	private $prefix = '_emsg_company_';

	public static function i18nForm($id) {
		return EMSG_i18n::getForm($id);
 }

	public static function i18nLabel($id) {
		return EMSG_i18n::getLabel($id);
	}

	public static function getListAsOptions() {
		$args = array(
			'post_type' => 'emsg_company'
		);
		$query = new WP_Query( $args );
		if (count($query->posts) === 0) {
			return [];
		} else {
			$i = 0;
			$results = [];
			while ($i < count($query->posts)) {
				$results[$query->posts[$i]->ID] = $query->posts[$i]->post_title;
				$i++;
			};

			return $results;
		}
	}

	public static function getRedirectService($id) {
		$r = get_post_meta($id, self::$sprefix . "redirectService", true);
		// Gets redirecting service
    if (null === $r || '' === $r) {
      $currentUrl = home_url() . "/emsgr/";
      $r = $currentUrl;
    }

    if (substr($r, -1) !== '/') {
      $r = $r . "/";
    }

		return $r;
	}

	public function __construct() {

		add_action( 'init', [ $this, 'register_post_type' ] );
		add_action( 'cmb2_admin_init', [ $this, 'meta_box' ] );
		add_filter( 'manage' . $this->prefix . 'posts_columns', [ $this, 'columns' ] );
		add_action( 'manage' . $this->prefix . 'posts_custom_column', [ $this, 'columns_content' ], 10, 2 );
	}

	public function log_mailer_errors( $wp_error ) {
		print_r($wp_error);
	}

	public function register_post_type() {

		$labels = [
			'name'               => self::i18nLabel('Companies'),
			'singular_name'      => self::i18nLabel('emsg_company'),
			'menu_name'          => self::i18nLabel('Companies'),
			'name_admin_bar'     => self::i18nLabel('Company'),
			'add_new'            => self::i18nLabel('Add New'),
			'add_new_item'       => self::i18nLabel('Add New Company'),
			'new_item'           => self::i18nLabel('New Company'),
			'edit_item'          => self::i18nLabel('Edit Company'),
			'view_item'          => self::i18nLabel('View Company'),
//			'all_items'          => self::i18nLabel('All Signatures',               'user-mail-signature-generator' ),
			'search_items'       => self::i18nLabel('Search Companies'),
			'parent_item_colon'  => self::i18nLabel('Parent Redirects:'),
			'not_found'          => self::i18nLabel('No companies found.'),
			'not_found_in_trash' => self::i18nLabel('No companies found in Trash.')
		];

		$args = [
			'labels'    => $labels,
			'public'    => true,
			'menu_icon' => 'dashicons-admin-links',
			'supports'  => [ 'title' ],
			'rewrite'   => [ 'slug' => $this->type, 'with_front' => false ],
			'show_ui'    => true,
			'show_in_menu' => false,
	    'show_in_nav_menus' => false
		];

		register_post_type( $this->type, $args );
	}

	public function add_field($cmb, $tab, $name, $id, $type, $desc = '') {
		EMSG_Plugin::add_field($this->prefix, $cmb, $name, $id, $type, $tab, self::i18nForm($desc));
	}

	public function meta_box() {

		$cmb = new_cmb2_box( [
			'id'           => $this->prefix . 'metabox',
			'title'        => self::i18nLabel('eMail Signature Manager Settings'),
			'object_types' => [ $this->type ],
			'tabs'      => array(
				'identity' => array(
					'label' => __( 'Identity'),
					//'show_on_cb' => 'yourprefix_show_if_front_page',
				),
				'contact' => array(
					'label' => __( 'Contact'),
				),
				'social'  => array(
					'label' => __( 'Social Media'),
					'icon'  => 'dashicons-share', // Dashicon
				),
				'message' => array(
					'label' => __( 'Messages'),
				),
				'disclaimer' => array(
					'label' => __( 'Disclaimer'),
				),
				'settings' => array(
					'label' => __( 'Settings'),
				)
			),
			'tab_style'   => 'classic'
		] );

		/* Identity */
		$this->add_field($cmb, 'identity', 'Company name', 'name', 'text');
		$this->add_field($cmb, 'identity', 'Logo image', 'logoSrc', 'file');
		$this->add_field($cmb, 'identity', 'Logo alt message', 'logoAltMessage', 'text');
//		$this->add_field($cmb, 'Logo target URL', 'logoTargetURL', 'text_url');
		$this->add_field($cmb, 'identity', 'Separator', 'separator', 'text');

		/* Address Contact */
		$this->add_field($cmb, 'contact', 'Adress 1', 'address1', 'text');
		$this->add_field($cmb, 'contact', 'Adress 2', 'address2', 'text');
		$this->add_field($cmb, 'contact', 'Zip', 'zip', 'text');
		$this->add_field($cmb, 'contact', 'City', 'city', 'text');
		$this->add_field($cmb, 'contact', 'Country', 'country', 'text');
		$this->add_field($cmb, 'contact', 'E-mail', 'mail', 'text');
		$this->add_field($cmb, 'contact', 'Website', 'website', 'text_url');

		/* Social */
		$this->add_field($cmb, 'social', 'Twitter id', 'twitter', 'text');
		$this->add_field($cmb, 'social', 'LinkedIn id', 'linkedin', 'text');
		$this->add_field($cmb, 'social', 'Facebook id', 'facebook', 'text');

		/* Disclaimer */
		$this->add_field($cmb, 'disclaimer', 'Disclaimer Text', 'disclaimer', 'textarea');

		/* Message */
		$this->add_field($cmb, 'message', 'Baseline', 'baseline', 'text');
		//$this->add_field($cmb, 'message', 'Banner image', 'bannerImage', 'file');
		$this->add_field($cmb, 'message', 'Banner alt message', 'bannerAlt', 'text');
		//$this->add_field($cmb, 'message', 'Banner target URL', 'bannerTargetURL', 'text_url');
		$this->add_field($cmb, 'message', 'Message1', 'message1', 'textarea');
		$this->add_field($cmb, 'message', 'Message2', 'message2', 'textarea');

		/* Parameters */
		$this->add_field($cmb, 'settings', 'Service de redirection', 'redirectService', 'text_url');

	}

	public function columns() {

		return [
			'cb'        => '<input type="checkbox" />',
			'title'     => self::i18nLabel('Title'),
			'name'      => self::i18nLabel('Company name'),
			'mail'      => self::i18nLabel('E-mail'),
			'website'   => self::i18nLabel('Website')
		];

	}

	public function columns_content( $column, $post_id ) {

		switch ( $column ) {

			case 'name' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'name', true ) );
				echo $p;
				break;

			case 'mail' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'mail', true ) );
				echo $p;
				break;

			case 'website' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'website', true ) );
				echo $p;
				break;
		}

	}


}

endif;

new EMSG_Company_Plugin();
