<?php

require_once plugin_dir_path( __FILE__ ) . 'class-emsg-signaturegenerator-plugin.php';

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'EMSG_Person_Plugin' ) ) :

class EMSG_Person_Plugin {

	private $type = 'emsg_person';
	private $prefix = '_emsg_person_';

	public static function i18nForm($id) {
		return EMSG_i18n::getForm($id);
 }

	public static function i18nLabel($id) {
		return EMSG_i18n::getLabel($id);
	}

	public function __construct() {

		add_action( 'init', [ $this, 'register_post_type' ] );
		add_action( 'cmb2_admin_init', [ $this, 'meta_box' ] );
		add_filter( 'manage' . $this->prefix . 'posts_columns', [ $this, 'columns' ] );
		add_action( 'manage' . $this->prefix . 'posts_custom_column', [ $this, 'columns_content' ], 10, 2 );
	}

	public function log_mailer_errors( $wp_error ) {
		print_r($wp_error);
	}

	public function register_post_type() {

		$labels = [
			'name'               => self::i18nLabel( 'EMSGS Persons' ),
			'singular_name'      => self::i18nLabel( 'emsg_person' ),
			'menu_name'          => self::i18nLabel( 'Signatures' ),
			'name_admin_bar'     => self::i18nLabel( 'Signature' ),
			'add_new'            => self::i18nLabel( 'Add New' ),
			'add_new_item'       => self::i18nLabel( 'Add New Mail Signature' ),
			'new_item'           => self::i18nLabel( 'New Mail Signature' ),
			'edit_item'          => self::i18nLabel( 'Edit Mail Signature' ),
			'view_item'          => self::i18nLabel( 'View Mail Signature' ),
//			'all_items'          => self::i18nLabel( 'All Signatures' ),
			'search_items'       => self::i18nLabel( 'Search Signatures' ),
			'parent_item_colon'  => self::i18nLabel( 'Parent Redirects:' ),
			'not_found'          => self::i18nLabel( 'No signatures found.' ),
			'not_found_in_trash' => self::i18nLabel( 'No signatures found in Trash.' )
		];

		$args = [
			'labels'    => $labels,
			'public'    => true,
			'menu_icon' => 'dashicons-admin-links',
			'supports'  => [ 'title' ],
			'rewrite'   => [ 'slug' => $this->type, 'with_front' => false ],
			'show_ui' 	=> true,
	    'show_in_menu' => false,
	    'show_in_nav_menus' => false
		];

		register_post_type( $this->type, $args );

	}

	public function add_field($cmb, $name, $id, $type, $tab) {
		EMSG_Plugin::add_field($this->prefix, $cmb, $name, $id, $type, $tab, '');
	}

	public function meta_box() {

		$cmb = new_cmb2_box( [
			'id'           => $this->prefix . 'metabox',
			'title'        => self::i18nLabel( 'eMail Signature Manager Settings'),
			'object_types' => [ $this->type ],
			'tabs'      => array(
				'identity' => array(
					'label' => __( 'Identity', 'cmb2' ),
					//'show_on_cb' => 'yourprefix_show_if_front_page',
				),
				'communication' => array(
					'label' => __( 'Communication', 'cmb2' ),
				),
				'social'  => array(
					'label' => __( 'Social Media', 'cmb2' ),
					'icon'  => 'dashicons-share', // Dashicon
				),
				'signature'  => array(
					'label' => __( 'Signature', 'cmb2' ),
					'icon'  => 'dashicons-share', // Dashicon
				),
			),
			'tab_style'   => 'classic'
		] );

		$cmb->add_field( [
			'name'      	=> self::i18nLabel( 'Company' ),
			'id'        	=> $this->prefix . 'company',
			'type'      	=> 'select',
			'options'			=> EMSG_Company_Plugin::getListAsOptions(),
	    'placeholder' => 'Select an Item',
			'attributes' 	=> [
				'required' 	=> 'required'
			]
		] );

		$this->add_field($cmb, 'Firstname', 'firstname', 'text', 'identity');
		$this->add_field($cmb, 'Lastname', 'lastname', 'text', 'identity');
		$this->add_field($cmb, 'Initials', 'initials', 'text', 'identity');
		$this->add_field($cmb, 'Photo image', 'photoSrc', 'file', 'identity');
		$this->add_field($cmb, 'Photo alt message', 'photoAlt', 'text', 'identity');
		$this->add_field($cmb, 'Photo target URL', 'photoHref', 'text_url', 'identity');

		$this->add_field($cmb, 'Email', 'mail', 'text', 'communication');
		$this->add_field($cmb, 'Job Title', 'jobtitle', 'text', 'communication');
		$this->add_field($cmb, 'Phone', 'phone', 'text', 'communication');
		$this->add_field($cmb, 'Mobile', 'mobile', 'text', 'communication');

		$this->add_field($cmb, 'Twitter', 'twitterId', 'text', 'social');
		$this->add_field($cmb, 'LinkedIn', 'linkedinId', 'text', 'social');
		$this->add_field($cmb, 'Facebook', 'facebookId', 'text', 'social');

		$this->add_field($cmb, 'Sincerely', 'sincerely', 'text', 'signature');

	}

	public function columns() {
		return [
			'cb'        => '<input type="checkbox" />',
			'title'     => self::i18nLabel( 'Title' ),
			'signature' => self::i18nLabel( 'Signature' ),
			'firstname' => self::i18nLabel( 'Firstname' ),
			'lastname'  => self::i18nLabel( 'Lastname' ),
			'mail'      => self::i18nLabel( 'E-Mail' )
		];

	}

	public function columns_content( $column, $post_id ) {

		switch ( $column ) {

			case 'firstname' :
				$firstname = esc_html( get_post_meta( $post_id, $this->prefix . 'firstname', true ) );
				echo $firstname;
				break;

			case 'lastname' :
				$lastname = esc_html( get_post_meta( $post_id, $this->prefix . 'lastname', true ) );
				echo $lastname;
				break;

			case 'mail' :
				$mail = esc_html( get_post_meta( $post_id, $this->prefix . 'mail', true ) );
				echo $mail;
				break;

			case 'signature' :
				$companyId = get_post_meta($post_id, $this->prefix . 'company', true);
				$companyMail = get_post_meta($companyId, '_emsg_company_mail', true);

				$url = esc_url( get_the_permalink( $post_id ) );
				$userMail = esc_html( get_post_meta( $post_id, $this->prefix . 'mail', true ) );

				$hide = self::i18nLabel( 'Hide' );
				$view = self::i18nLabel( 'View' );
				$signature = EMSG_SignatureGenerator_Plugin::execute($post_id);
				?>

				<a href='/emsgr/send-signature/<?php echo $companyMail; ?>/<?php echo $userMail; ?>'>Send signature</a>
				<button id="button-<?php echo $post_id; ?>" type="button" class="emsg-copy button button-small" onclick="toggle_view_hide('<?php echo $post_id; ?>', '<?php echo $view; ?>', '<?php echo $hide; ?>' )">
					<?php esc_html_e( 'View', 'user-mail-signature-generator' ); ?>
				</button>

				<div id="preview-<?php echo $post_id; ?>" style="background-color: white; display: none;">
					<div style="margin: 10px; border: 1px solid silver; padding: 10px">
						<?php echo $signature; ?>
				  </div>
				</div>

				<?php

				break;

		}

	}

}

endif;

new EMSG_Person_Plugin();
