<?php

require_once plugin_dir_path( __FILE__ ) . './vendor/Parsedown.php';

include( plugin_dir_path( __FILE__ ) . 'class-emsg-i18n-plugin.php');
include( plugin_dir_path( __FILE__ ) . 'class-emsg-redirect-plugin.php');
include( plugin_dir_path( __FILE__ ) . 'class-emsg-company-plugin.php');
include( plugin_dir_path( __FILE__ ) . 'class-emsg-person-plugin.php');
include( plugin_dir_path( __FILE__ ) . 'class-emsg-signaturegenerator-plugin.php');

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'EMSG_Plugin' ) ) :

class EMSG_Plugin {

	public function __construct() {

		add_action( 'plugins_loaded',  [ $this, 'include_vendor' ] );
		add_action( 'init', [ $this, 'register_post_type' ] );

		add_action( 'admin_enqueue_scripts', [ $this, 'admin_assets' ] );
		add_action( 'admin_notices', [ $this, 'pretty_permalinks_notice' ] );
		add_action( 'template_redirect', [ $this, 'do_redirect' ] );
		add_action( 'admin_menu', [$this, 'mail_signature_plugin_setup_menu'] );
		add_action( 'save_post', [$this, 'change_title']);

	}

	public function change_title($post_id) {
		//print_r($_POST);
		if (! array_key_exists('post_type', $_POST)) {
/*
			print("POST TYPE=" . $_POST['post_type']);
			if ('emsg_company' === $_POST['post_type']
					|| 'emsg_person' === $_POST['post_type']
					|| 'emsg_redirect' === $_POST['post_type']) {
			} else {
				return;
			}
*/
			return;
		}

		$args = [];
		$post_title = EMSG_Plugin::getKeyOrEmpty('post_title', $_POST);
	  switch ($_POST['post_type']) {
			case 'emsg_company':
					$name = EMSG_Plugin::getKeyOrEmpty('_emsg_company_name', $_POST);
					$mail = EMSG_Plugin::getKeyOrEmpty('_emsg_company_mail', $_POST);

					$post_title = $name . " / " . $mail;
			  break;
			case 'emsg_person':
					$lastname = EMSG_Plugin::getKeyOrEmpty('_emsg_person_lastname', $_POST);
					$firstname = EMSG_Plugin::getKeyOrEmpty('_emsg_person_firstname', $_POST);
					$initials = EMSG_Plugin::getKeyOrEmpty('_emsg_person_initials', $_POST);

					$post_title = $lastname . " / " . $firstname . " / " . $initials;
				break;
			case 'emsg_redirect':
					$companyId = EMSG_Plugin::getKeyOrEmpty('_emsg_redirect_company', $_POST);
					$company = get_post_meta($companyId, '_emsg_company_name', true );
					$key = get_post_meta($companyId, '_emsg_company_mail', true );
					$type = EMSG_Plugin::getKeyOrEmpty('_emsg_redirect_type', $_POST);
//					$key = EMSG_Plugin::getKeyOrEmpty('_emsg_redirect_key', $_POST);

					$args['_emsg_redirect_key'] = $key;
					$post_title = $company . " / " . $type	. " / " . $key;
				break;
		}
		$args['ID'] = $post_id;
		$args['post_title'] = $post_title;

		// unhook so it doesn't loop infinitely
		remove_action('save_post', [$this, 'change_title']);
		// update the post, which calls save_post again
		wp_update_post($args);
		//update_post_meta($post_id, '_emsg_redirect_key', $key);
		// re-hook this function
		add_action('save_post', [$this, 'change_title']);

	}

	function mail_signature_plugin_setup_menu() {
	      add_menu_page('eMail Signature Plugin Page', 'eMail Signature Manager', 'manage_options',
					'email-signature-plugin', [$this, 'mail_signature_init']);
				add_submenu_page('email-signature-plugin', 'eMail Signature Manager Plugin', 'eMail Signature Manager', 'manage_options', 'email-signature-plugin');

				/* Redirects */
				add_submenu_page('email-signature-plugin', 'List all redirects', 'List all redirects', 'manage_options',
					'./edit.php?post_type=emsg_redirect', '');
				add_submenu_page('email-signature-plugin', 'Add item', '-> Add redirect', 'manage_options',
					'./post-new.php?post_type=emsg_redirect', '');

				/* Companies */
				add_submenu_page('email-signature-plugin', 'List all companies', 'List all companies', 'manage_options',
					'./edit.php?post_type=emsg_company', '');
				add_submenu_page('email-signature-plugin', 'Add item', '-> Add company', 'manage_options',
					'./post-new.php?post_type=emsg_company', '');

				/* Signatures */
				add_submenu_page('email-signature-plugin', 'List all persons (signatures)', 'List all persons (signatures)', 'manage_options',
					'./edit.php?post_type=emsg_person', '');
				add_submenu_page('email-signature-plugin', 'Add item', '-> Add person (signature)', 'manage_options',
					'./post-new.php?post_type=emsg_person', '');

//				add_submenu_page('mail-signature-plugin', 'Company', 'Company', 'manage_options', 'company', 'execute');
//				add_submenu_page('./post-new.php?post_type=emsg', 'Company Page', 'Company', 'manage_options', 'company', '');
	}

	public function mail_signature_init() {
		$Parsedown = new Parsedown();
		$file = file_get_contents(plugin_dir_path( __FILE__ ) . '../assets/documentation.md');
		echo $Parsedown->text($file);
	}

	public function include_vendor() {
		include EMSG_PATH . 'includes/vendor/cmb2/init.php';
		include EMSG_PATH . 'includes/vendor/cmb2-conditionals/cmb2-conditionals.php';
		include EMSG_PATH . 'includes/vendor/cmb2-tabs/cmb2-tabs.php';
	}

	public function register_post_type() {
	}

	public function meta_box() {
	}

	public function add_sortable_visits_column( $columns ) {
		$columns['emsg_visits'] = 'emsg_visits';
		return $columns;
	}

	public function sortable_visits_content( $query ) {

		if ( ! is_admin() ) {
			return;
		}

		if ( $query->is_main_query() && 'emsg_visits' === $query->get( 'orderby' ) ) {

			$query->set( 'meta_query', [
				'relation' => 'OR',
				[
					'key'     => '_emsg_visits',
					'compare' => 'EXISTS',
				],
				[
					'key'     => '_emsg_visits',
					'compare' => 'NOT EXISTS',
				],
			] );

			$query->set( 'orderby', 'meta_value_num date' );

		}

	}

	public function admin_assets() {

		$screen = get_current_screen();

//		if ('emsg_person' === $screen->post_type
//						|| 'emsg_company' === $screen->post_type) {

			wp_enqueue_style(
				'emsg-admin-css',
				esc_url( EMSG_URL . 'assets/css/emsg-admin.css' ),
				array(),
				EMSG_VERSION
			);

			wp_enqueue_script(
				'emsg-admin-js',
				esc_url( EMSG_URL . 'assets/js/emsg-admin.js' ),
				array(),
				EMSG_VERSION,
				false
			);

			wp_localize_script( 'emsg-admin', 'emsgAdminL10n', [
				'promptText' => esc_attr__( "Here's the URL:", 'email-signature-generator' ),
			] );
//		}

	}

	public function pretty_permalinks_notice() {

		if ( get_option( 'permalink_structure' ) ) {
			return;
		}

		$link = sprintf(
			'<a href="%s">%s</a>',
			esc_url( admin_url( 'options-permalink.php' ) ),
			esc_html__( 'Permalink Settings', 'email-signature-generator' )
		);

		$message = sprintf( esc_html__( 'eMail Signature Manager is not compatible with Plain Permalinks. Please choose a different setting in %s.', 'email-signature-generator' ), $link );
		$message = sprintf( '<div class="error">%s</div>', wpautop( $message ) );

		echo wp_kses_post( $message );

	}

	public static function add_field($prefix, $cmb, $name, $id, $type, $tab, $desc = '') {
		$args = [
					'name' => esc_html__($name, 'user-mail-signature-generator'),
					'id'   => $prefix . $id,
					'desc' => $desc,
					'type' => $type,
		];
		if ($tab !== '') {
				$args['tab'] = $tab;
				$args['render_row_cb'] = array( 'CMB2_Tabs', 'tabs_render_row_cb' );
		}

		$cmb->add_field($args);
	}

	public static function getKeyOrEmpty($key, $arr) {
		return array_key_exists($key, $arr) ? $arr[$key] : '';
	}

}

endif;

new EMSG_Plugin();
