<?php

require_once plugin_dir_path( __FILE__ ) . 'class-log.php';
require_once plugin_dir_path( __FILE__ ) . 'class-emsg-signaturegenerator-plugin.php';

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'EMSG_Redirect_Plugin' ) ) :

class EMSG_Redirect_Plugin {

	private $slug = 'emsgr';
	private $type = 'emsg_redirect';
	private $prefix = '_emsg_redirect_';

	public static function i18nForm($id) {
		return EMSG_i18n::getForm($id);
 }

	public static function i18nLabel($id) {
		return EMSG_i18n::getLabel($id);
	}

	public function __construct() {

		add_action( 'init', [ $this, 'register_post_type' ] );
		add_action( 'cmb2_admin_init', [ $this, 'meta_box' ] );
		add_filter( 'manage' . $this->prefix . 'posts_columns', [ $this, 'columns' ] );
		add_action( 'manage' . $this->prefix . 'posts_custom_column', [ $this, 'columns_content' ], 10, 2 );
		add_action( 'template_redirect', [ $this, 'do_redirect' ] );
		add_action( 'wp_mail_failed', [ $this, 'log_mailer_errors' ], 10, 1);
	}

	public function log_mailer_errors( $wp_error ) {
		print_r($wp_error);
	}

	public function register_post_type() {

		$labels = [
			'name'               => self::i18nLabel( 'EMSG Redirect' ),
			'singular_name'      => self::i18nLabel( 'emsg_redirect' ),
			'menu_name'          => self::i18nLabel( 'Redirects' ),
			'name_admin_bar'     => self::i18nLabel( 'Redirect' ),
			'add_new'            => self::i18nLabel( 'Add New' ),
			'add_new_item'       => self::i18nLabel( 'Add New Redirect' ),
			'new_item'           => self::i18nLabel( 'New Redirect' ),
			'edit_item'          => self::i18nLabel( 'Edit Redirect' ),
			'view_item'          => self::i18nLabel( 'View Redirect' ),
//			'all_items'          => self::i18nLabel( 'All EMSG Redirects' ),
			'search_items'       => self::i18nLabel( 'Search Redirects' ),
			'parent_item_colon'  => self::i18nLabel( 'Parent Redirects:' ),
			'not_found'          => self::i18nLabel( 'No redirects found.' ),
			'not_found_in_trash' => self::i18nLabel( 'No redirects found in Trash.' )
		];

		$args = [
			'labels'    => $labels,
			'public'    => true,
			'public_queryable' => true,
    	'query_var'         => true,
			'menu_icon' => 'dashicons-admin-links',
			'supports'  => [ 'title' ],
			'rewrite'   => [ 'slug' => $this->slug, 'with_front' => false ],
			'show_ui'    => true,
			'show_in_menu' => false,
	    'show_in_nav_menus' => false
		];

    //remove_post_type_support($this->type, 'title');
		register_post_type( $this->type, $args );
	}

	public function add_field($cmb, $name, $id, $type, $tab) {
		EMSG_Plugin::add_field($this->prefix, $cmb, $name, $id, $type, $tab);
	}

	public function meta_box() {
		$cmb2_conditionals = new CMB2_Conditionals();

		$cmb = new_cmb2_box( [
			'id'           => $this->prefix . 'metabox',
			'title'        => self::i18nLabel( 'eMail Signature Manager Settings' ),
			'object_types' => [ $this->type ],
			'tabs'      => array(
				'topic' => array(
					'label' => __( 'Communication Topic'),
					//'show_on_cb' => 'yourprefix_show_if_front_page',
				),
				'resource'  => array(
					'label' => __( 'Resource'),
					'icon'  => 'dashicons-share', // Dashicon
				),
				'validity'  => array(
					'label' => __( 'Validity'),
					'icon'  => 'dashicons-share', // Dashicon
				)
			),
			'tab_style'   => 'classic'
		] );

		$cmb->add_field( [
			'name'      	=> self::i18nLabel( 'Company' ),
			'id'        	=> $this->prefix . 'company',
			'type'      	=> 'select',
			'options'			=> EMSG_Company_Plugin::getListAsOptions(),
	    'placeholder' => 'Select an Item',
			'attributes' 	=> [
				'required' 	=> 'required',
			],
			'tab' => 'topic',
			'render_row_cb' => [ 'CMB2_Tabs', 'tabs_render_row_cb' ]
		] );

		$this->add_field($cmb, 'Communication Name', 'name', 'text', 'topic');

		/* $this->add_field($cmb, 'Communication Mail Key', 'key', 'text_email', ''); */
		$cmb->add_field( array(
			'name'       => 'Communication Mail Key',
			'id'         => $this->prefix . 'key',
			'type'       => 'text_email',
			'attributes' => [
				'required'            => false, // Will be required only if visible.
				'data-conditional-id' => $this->prefix . 'company'
			],
			'tab' => 'topic',
			'render_row_cb' => ['CMB2_Tabs', 'tabs_render_row_cb']
		) );
		$this->add_field($cmb, 'User Mail Key', 'usermail', 'text_email', 'topic');

		$cmb->add_field( [
			'name'      	=> self::i18nLabel( 'Communication Type' ),
			'id'        	=> $this->prefix . 'type',
			'type'      	=> 'select',
			'options'			=>
				[
					"r-i-ba" => "Image redirection for banner",
					"r-i-lo" => "Image redirection for logo",
					"r-i-ph" => "Image redirection for user photo",
					"r-h-ba" => "Link redirection for banner",
					"r-h-lo" => "Link redirection for logo",
					"r-h-ph" => "Link redirection for user photo",
					"r-so-ph" => "Link redirection for twitter",
					"r-so-ph" => "Link redirection for linkedin",
					"r-so-ph" => "Link redirection for facebook",
				],
	    'placeholder' => 'Select an Item',
			'attributes' 	=> [
				'required' 	=> 'required',
			],
			'tab' => 'resource',
			'render_row_cb' => ['CMB2_Tabs', 'tabs_render_row_cb']
		] );
		$cmb->add_field( array(
			'name'       => 'Image',
			'id'         => $this->prefix . 'src',
			'type'       => 'file',
			'attributes' => array(
				'required'            => true, // Will be required only if visible.
				'data-conditional-id' => $this->prefix . 'type',
				'data-conditional-value' => wp_json_encode( ['r-i-ba', 'r-i-lo', 'r-i-ph'] )
			),
			'tab' => 'resource',
			'render_row_cb' => ['CMB2_Tabs', 'tabs_render_row_cb']
		) );

		/* $this->add_field($cmb, 'Target URL', 'href', 'text_url', ''); */
		$cmb->add_field( array(
			'name'       => 'Target URL',
			'id'         => $this->prefix . 'target',
			'type'       => 'text_url',
			'attributes' => [
				'required'            => true, // Will be required only if visible.
				'data-conditional-id' => $this->prefix . 'type',
				'data-conditional-value' => wp_json_encode( ['r-h-ba', 'r-h-lo', 'r-h-ph',
																											'r-so-tw', 'r-so-li', 'r-so-fa'] )
			],
			'tab' => 'resource',
			'render_row_cb' => ['CMB2_Tabs', 'tabs_render_row_cb']
		) );

		$cmb->add_field( [
			'name'       => self::i18nLabel( 'Visits' ),
			'id'         => $this->prefix . 'visits',
			'type'       => 'text_small',
			'attributes' => [
				'readonly' => 'readonly',
				'value'    => isset( $_GET['post'] ) ? (int) get_post_meta( $_GET['post'], '_emsg_redirect_visits', true ) : 0,
			],
			'tab' => 'resource',
			'render_row_cb' => ['CMB2_Tabs', 'tabs_render_row_cb']
		] );

		$this->add_field($cmb, 'Start', 'start', 'text_datetime_timestamp', 'validity');
		$this->add_field($cmb, 'End', 'end', 'text_datetime_timestamp', 'validity');
		$this->add_field($cmb, 'Enabled', 'enabled', 'checkbox', 'validity');

	}

	public function columns() {

		return [
			'cb'        => '<input type="checkbox" />',
			'title'     => self::i18nLabel( 'Title' ),
			'name'      => self::i18nLabel( 'Name' ),
			'type'      => self::i18nLabel( 'Type' ),
			'key'   		=> self::i18nLabel( 'Key' ),
//			'usermail'  => self::i18nLabel( 'User mail key' ),
			'src_target'			=> self::i18nLabel( 'Image/URL' ),
			'visits'		=> self::i18nLabel( 'Visits' ),
			'period'		=> self::i18nLabel( 'Period' ),
			'enabled'		=> self::i18nLabel( 'Enabled' )
		];
	}

	public function columns_content( $column, $post_id ) {

		switch ( $column ) {
/*
			case 'title' :
				break;
*/
			case 'name' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'name', true ) );
				echo $p;
				break;

			case 'key' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'key', true ) );
				echo $p;
				break;

			case 'type' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'type', true ) );
				echo $p;
				break;
/*
			case 'usermail' :
				$p = esc_html( get_post_meta( $post_id, $this->prefix . 'usermail', true ) );
				echo $p;
				break;
*/
			case 'src_target' :
				$src = esc_html( get_post_meta( $post_id, $this->prefix . 'src', true ) );
				if ( "" !== $src) {
					//@TODO
					//$href = $redirectService/$type/$key/$user
					$companyId = get_post_meta( $post_id, $this->prefix . 'company', true );
					$redirectService = EMSG_Company_Plugin::getRedirectService($companyId);
					$type = "r-h-ba";
					$key = get_post_meta( $post_id, $this->prefix . 'key', true );
					$usermail = get_post_meta( $post_id, $this->prefix . 'usermail', true );

					// $redirectService ends with a "/"
					$href = $redirectService . $type . "/" . $key . "/" . $usermail;
					echo "<a href='" . $href . "' target='blank'>";
					echo "<img src='" . $src . "' width='80px'>";
					echo "</a>";
				} else {
					$href = esc_html( get_post_meta( $post_id, $this->prefix . 'target', true ) );
					echo "<a href='" . $href . "' target='blank'>";
					echo $href;
					echo "</a>";
				}
				break;

			case 'visits' :
				echo '<span class="redirect_count">'
							. number_format_i18n(absint(get_post_meta($post_id, '_emsg_redirect_visits', true )))
						.'</span>';
				break;

			case 'period' :
				$start = get_post_meta($post_id, '_emsg_redirect_start', true);
				$end = get_post_meta($post_id, '_emsg_redirect_end', true);
				$date1 = $start !== "" ? gmdate("Y-m-d\TH:i:s\Z", $start) : "Not indicated";
				$date2 = $end !== "" ? gmdate("Y-m-d\TH:i:s\Z", $end) : "Not indicated";
				$now = date('U');
				if ($start <= $now && $now <= $end) {
					$cssClass = "period_active";
				} else {
					$cssClass = "period_inactive";
				}
				echo '<span class="period ' . $cssClass . '">'
							. $date1
							. "<br/> -> "
							. $date2
						.'</span>';
				break;

			case 'enabled' :
				echo '<span class="enabled">'
						. get_post_meta($post_id, '_emsg_redirect_enabled', true)
						.'</span>';
				break;

		}

	}

	public function getCompanyId($key, $user) {
		$args = array(
			'post_type' => 'emsg_company',
			'meta_query' => array (
				array (
					'key' => '_emsg_company_mail', 'compare' => '=', 'value' => $key
				)
			)
		);
		$query = new WP_Query($args);
		if (0 == $query->post_count) {
			return null;
		} else {
			return $query->post->ID;
		}
	}

	public function getPersonId($key, $user) {
		// $key is not used for the moment
		$args = array(
			'post_type' => 'emsg_person',
			'meta_query' => array (
				array (
					'key' => '_emsg_person_mail', 'compare' => '=', 'value' => $user
				)
			)
		);
		$query = new WP_Query($args);
		if (0 == $query->post_count) {
			return null;
		} else {
			return $query->post->ID;
		}
	}

	public function getRedirectId($route, $key, $user) {
		$now = date('U');
		$args = array(
			'post_type' => 'emsg_redirect',
			//'title' => $route,
			'meta_query' => array (
				array (
					'key' => '_emsg_redirect_type', 'compare' => '=', 'value' => $route
				),
				array (
					'key' => '_emsg_redirect_key', 'compare' => '=', 'value' => $key
				),
				array (
					'key' => '_emsg_redirect_enabled', 'compare' => '=', 'value' => 'on'
				),
				array(
          'key' => $this->prefix . 'start', 'compare' => '<=', 'value' => $now
        ),
        array(
					'key' => $this->prefix . 'end', 'compare' => '>=', 'value' => $now
        )
			)
		);

		if (null !== $user) {
			// If user not null, we add the usermail criteria to the query args
			$args["meta_query"][2] = array (
				'key' => '_emsg_redirect_usermail', 'compare' => '=', 'value' => $user
			);
		}

		$query = new WP_Query($args);
		// Useful to check if search query is ok
		//print_r($query);
		if (0 == $query->post_count) {
			return null;
		} else {
			Log::debug("### Computing best postId...");
			$visits = 0;
			$count = $query->post_count;
			for ($i = 0; $i < $count; ++$i) {
				//$visits = 100;
				$postId = $query->posts[$i]->ID;
				$visits1 = (int) get_post_meta($postId, '_emsg_redirect_visits', true);
				Log::debug("i: $i / Post id: $postId / Visits: $visits1");
				$visits += $visits1;
			}
			$j = $visits % $count;
			$fPostId = $query->posts[$j]->ID;
			Log::debug("### $visits % $count = $j => Best postId = posts[$j]->ID = " . $fPostId);
			//return $query->post->ID;
			return $fPostId;
		}
	}

	public function getRedirectIdByCascade($route, $key, $user) {

		$postId = $this->getRedirectId($route, $key, $user);
		if (null !== $postId) {
			return $query->post->ID;
		} else {
			// Redirect default search with $user fails so we try without it
			$postId = $this->getRedirectId($route, $key, null);
			if (null !== $postId) {
				return $postId;
			} else {
				return null;
			}
		}
	}

	public function getUrl($postId, $urlType, $entityType, $key, $user) {
		// It's a redirect
		if (null !== $postId) {
			Log::debug("Post Id (redirect): " . $postId);
			if ('href' === $urlType || 'h' === $urlType) {
				// It's a href link
				$url = esc_html(get_post_meta( $postId, '_emsg_redirect_target', true ));
			} else {
				// it's an image
				$url = esc_html(get_post_meta( $postId, '_emsg_redirect_src', true ));
			}
		} else {
			// No redirect. An entity resource must be available
			if ('person' === $entityType) {
				$postId = $this->getPersonId($key, $user);
				Log::debug("Post Id (person): " . $postId);
				if ('href' === $urlType || 'h' === $urlType) {
					// It's a href link
					$url = esc_html(get_post_meta( $postId, '_emsg_person_photoHref', true ));
				} else {
					// It's an image
					$url = esc_html(get_post_meta( $postId, '_emsg_person_photoSrc', true ));
				}
			} else {
				// 'company' === $entityType
				$postId = $this->getCompanyId($key, $user);
				Log::debug("Post Id (company): " . $postId);
				if ('href' === $urlType || 'h' === $urlType) {
					// It's a href link
					$url = esc_html(get_post_meta( $postId, '_emsg_company_website', true ));
				} else {
					// It's an image
					$url = esc_html(get_post_meta( $postId, '_emsg_company_logoSrc', true ));
				}
			}
		}

		return $url;
	}

	public function do_redirect() {
		global $wp;
    global $wp_query;

		$currentUrl = home_url(add_query_arg($_GET,$wp->request));
		$queryString = substr($currentUrl, strlen(get_home_url()));
		$keywords = preg_split("/\/+/", $queryString);

		$path = $keywords[1];
		$route = $keywords[2];
		$key = $keywords[3];
		$user = null;
		if (5 > count($keywords)) {
			$user = "nobody@company.com"; // default value
		} else {
			$user = $keywords[4];
		}

//		if ( ! is_singular( 'emsg_redirect' ) ) {
		if ( "emsgr" !== $path && "umsgr" !== $path) {
			return;
		}

		Log::debug("Route: " . $route);
		Log::debug("Key: " . $key);
		Log::debug("User mail: " . $user);

		$twitterHref = "https://twitter.com/";
	  $linkedinHref = "http://us.linkedin.com/in/";
	  $facebookHref = "https://www.facebook.com/";

		$url = "";
		switch ( $route ) {

			case 'r-so-tw' :
				$postId = $this->getPersonId($key, $user);
				Log::debug("Person redirect (twitter): " . $postId);
				$url = $twitterHref . esc_html(get_post_meta( $postId, '_emsg_person_twitterId', true ));
				break;

			case 'r-so-li' :
				$postId = $this->getPersonId($key, $user);
				Log::debug("Person redirect (linkedin): " . $postId);
				$url = $linkedinHref . esc_html(get_post_meta( $postId, '_emsg_person_linkedinId', true ));
				break;

			case 'r-so-fa' :
				$postId = $this->getPersonId($key, $user);
				Log::debug("Person redirect (facebook): " . $postId);
				$url = $facebookHref . esc_html(get_post_meta( $postId, '_emsg_person_facebookId', true ));
				break;

			case 'r-h-ph' :
				$postId = $this->getRedirectId($route, $key, $user);
				$url = $this->getUrl($postId, 'href', 'person', $key, $user);

				break;

			case 'r-h-lo' :
				$postId = $this->getRedirectIdByCascade($route, $key, $user);
				$url = $this->getUrl($postId, 'href', 'company', $key, $user);

				break;

			case 'r-h-we' :
				$postId = $this->getRedirectIdByCascade($route, $key, $user);
				$url = $this->getUrl($postId, 'href', 'company', $key, $user);

				break;

			case 'r-h-ba' :
				$postId = $this->getRedirectIdByCascade($route, $key, $user);
				Log::debug("Post Id (redirect) href: " . $postId);
				$url = esc_html( get_post_meta( $postId, '_emsg_redirect_target', true ) );
				break;

			case 'r-i-ph' :
				$postId = $this->getRedirectId($route, $key, $user);
				$url = $this->getUrl($postId, 'image', 'person', $key, $user);

				break;

			case 'r-i-lo' :
				$postId = $this->getRedirectIdByCascade($route, $key, $user);
				$url = $this->getUrl($postId, 'image', 'company', $key, $user);

				break;

			case 'r-i-ba' :
				$postId = $this->getRedirectIdByCascade($route, $key, $user);
				Log::debug("Post Id banner: " . $postId);
				$url = esc_html( get_post_meta( $postId, '_emsg_redirect_src', true ) );
				break;

			case 'send-signature' :
				$to = $user;
				$subject = 'Your signature';
				$body = 'Copy and paste this following signature in your signature mailer:';
				$headers = array('Content-Type: text/html; charset=UTF-8');
				$postId = $this->getPersonId($key, $user);
				Log::debug("Post Id photo (person): " . $postId);
				$message = EMSG_SignatureGenerator_Plugin::execute($postId);
				wp_mail( $to, $subject, $body . " - " . $message, $headers );
				Log::debug("Sending mail to: " . $to . " - " . $subject);
				$url = "/wp-admin/edit.php?post_type=emsg_person";
				break;
		}

		if ( $url ) {
			$visits = absint( get_post_meta( $postId, '_emsg_redirect_visits', true ) );
			update_post_meta( $postId, '_emsg_redirect_visits', $visits < 1 ? 1 : ++$visits );

			Log::debug("URL: " . $url);

			wp_redirect( $url, '302' );
			exit;
		} else {
			wp_die( self::i18nLabel( 'Redirect URL not found.' ) );
		}
	}

}

endif;

new EMSG_Redirect_Plugin();
